from flask import flash
from requests.auth import HTTPBasicAuth
import logging
import APIFactory
from providers import utils
from providers.utils import User
from providers import requestUtils
from providers.notifications import NotificationHandler


__author__ = 'hanl'


class Provider(object):
    def __init__(self, auth_header, config):
        self.error = False
        self.auth_header = auth_header.__name__
        self.type = None
        self.secret = config.get('SECRET_KEY', None)
        self.admins = config.get('ADMINS')[self.__class__.__name__]


    def authenticate(self, username=None, password=None):
        pass

    def get_user(self, session=None, full=False):
        if not session and not session['user_id']:
            return ValueError("username and session must be provided!")

    def login(self, session=None, user=None):
        pass

    # deprecated
    def is_error(self):
        if self.error:
            self.error = False
            return True
        return self.error

    # deprecated
    def is_admin(self, reference):
        '''
        sub classes should override and call this function and give it the
        respective reference, e.g token or a username or whatever reference should be
        used to identify admins
        :param reference:
        :return:
        '''
        return False

    def logout(self, session=None):
        pass

    def is_authenticated(self):
        '''
        check tokens for non expired status

        :return:
        '''
        pass

    def get_header(self, session):
        token = session[self.type]
        obj = getattr(__import__(requestUtils), self.auth_header, None)
        print "the instance: %s" % str(obj)
        return obj(token)


class CustomProvider(Provider):
    def __init__(self, config):
        super(CustomProvider, self).__init__(requestUtils.CustomAuth, config)
        self.type = 'api_token'

    def authenticate(self, username=None, password=None):
        pass

    def get_user(self, session=None, full=False):
        """
        Returns the user model instance associated with the given request session.
        If no user is retrieved no user instance is returned
        """

        if not session or not session['user_id']:
            return None
        if full:
            id_token = session[self.type]
            code = utils.decrypt_openid(secret=self.secret, token=id_token)
            if "email" in code and "firstName" in code and "lastName" in code:
                user = User(username=session['user_id'], email=code['email'], firstName=code['firstName'],
                            lastName=code['lastName'], address=code['address'], institution=code['institution'],
                            phone=code['phone'])
                return user
            else:
                NotificationHandler.notify({"error": "Data could not be loaded!"}, flash)
        return User(username=session['user_id'])


    def get_token(self, session, full=False):
        if full:
            id_token = session[self.type]
            code = utils.decrypt_openid(secret=self.secret, token=id_token)
            if "email" in code and "firstName" in code and "lastName" in code:
                user = User(username=session['user_id'], email=code['email'], firstName=code['firstName'],
                            lastName=code['lastName'], address=code['address'], institution=code['institution'],
                            phone=code['phone'])
                return user
            else:
                NotificationHandler.notify({"error": "Data could not be loaded!"}, flash)

    def login(self, session=None, user=None):
        '''
        :param login_func: client specific login function
        :param user: user object to register user for
        :return: boolean if login successful
        '''
        super(CustomProvider, self).login(session, user)

        response = APIFactory.get("auth/requestToken", auth=HTTPBasicAuth(username=user.username,
                                                                          password=user.password))
        user.password = None

        if response is None:
            return False
        elif NotificationHandler.isError(response):
            NotificationHandler.notify(response.json(), flash)
            return False
        print "the response %i:%s" % (response.status_code, str(response.content))
        if 'token_type' in response.content:
            json = response.json()
            session[self.type] = json['id_token']
        else:
	    token = response.content.replace(self.type, '')
            session[self.type] = token.replace(' ', '')
	logging.info("the resulting authentication token %s" % session[self.type])
        return True


    def logout(self, session=None):
        if self.type not in session:
            return False
        session.pop(self.type, None)
        return True

    def is_authenticated(self):
        '''
        check that oauth id_token and access_token are not expired!
        :return:
        '''
        pass


class OAuth2Provider(Provider):
    def __init__(self, config):
        super(OAuth2Provider, self).__init__(requestUtils.Oauth2Auth)
        self.type = 'access_token'
        self.client = config.get("OAUTH2_CLIENT_ID", "")
        self.secret = config.get("OAUTH2_CLIENT_SECRET", "")
        self.scopes = config.get("OPENID_CONNECT_SCOPES", "")

    def authenticate(self, username=None, password=None):
        pass

    def get_user(self, session=None, full=False):
        """
        Returns the user model instance associated with the given request session.
        If no user is retrieved None is returned
        """
        if not session or not session['user_id']:
            return None

        if full and "openid" in self.scopes and "profile" in self.scopes:
            id_token = session['id_token']
            code = utils.decrypt_openid(secret=self.secret, token=id_token)
            user = User(username=session['user_id'], email=code['email'], firstName=code['firstName'],
                        lastName=code['lastName'], address=code['address'], institution=code['institution'],
                        phone=code['phone'])
            return user
        elif full:
            response = APIFactory.get("user/info", auth=self.get_header(session[self.type]))
            if response is None:
                return None
            elif NotificationHandler.isError(response):
                NotificationHandler.notify(response.json(), flash)
                return None
            else:
                code = response.json()
                user = User(username=session['user_id'], email=code['email'], firstName=code['firstName'],
                            lastName=code['lastName'], address=code['address'], institution=code['institution'],
                            phone=code['phone'])
                return user
        else:
            # for the most tasks its only about to have a user object, not the actual data!
            return User(username=session['user_id'])

    def login(self, session=None, user=None):
        '''
        :param login_func: client specific login function
        :param user: user object to register user for
        :return: boolean if login successful
        '''
        super(OAuth2Provider, self).login(session, user)

        params = {"username": user.username, "password": user.password,
                  "grant_type": "password", "client_id": self.client,
                  "client_secret": self.secret,
                  "scope": self.scopes}
        response = APIFactory.post(path='oauth2/token', params=params)
        user.password = None

        if response is None:
            return False
        elif NotificationHandler.isError(response):
            NotificationHandler.notify(response.json(), flash)
            return False
        print "the response %i:%s" % (response.status_code, str(response.content))

        session[self.type] = response.json()[self.type]
        if "openid" in self.scopes:
            session['id_token'] = response.json()['id_token']
        else:
            session['id_token'] = ""
        return True

    def is_admin(self, session):
        if session['access_token'] in self.admins:
            return True

    def logout(self, session=None):
        if self.type not in session:
            return False

        session.pop(self.type, None)
        if 'id_token' in session:
            session.pop('id_token', None)
        return True


    def is_authenticated(self):
        '''
        check that oauth id_token and access_token are not expired!
        add function to auth decorator
        :return:
        '''
        pass
